<?php
namespace MRU\Revslider\Controller;

use MRU\Revslider\Domain\Helper\Proxy;
use MRU\Revslider\Repository\OptionsRepository;
use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Configuration\ConfigurationManager;
use TYPO3\CMS\Core\FormProtection\FormProtectionFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Manfred Rutschmann <manfred@rutschmann.biz>, Rutschmann.BIZ
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
class BackendController extends ActionController
{

    /**
     * @var \MRU\Revslider\Domain\Repository\OptionsRepository
     * @inject
     */
    protected $optionsRepository = NULL;

    /**
     * action show
     *
     * @return string
     */
    public function indexAction()
    {


        if(!file_exists(GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/system/core/CodeIgniter.php'))){
            $this->redirect('index', 'Install');
            return false;
        }

        $this->doFirstInstall();
        return $this->instantiateCi();
    }

    /**
     * @throws \CannotPerformOperationException
     */
    private function doFirstInstall()
    {
        $rows = BackendUtility::getRecordsByField('tx_revslider_domain_model_user', 'id', '1');
        if(count($rows) === 0){
            $sql = "CREATE TABLE tx_revslider_domain_model_options (
                                              uid int(11) DEFAULT '0' NOT NULL,
                                              pid int(11) DEFAULT '0' NOT NULL,
                                              id int(9) NOT NULL auto_increment,
                                              handle varchar(100) DEFAULT '' NOT NULL,
                                              `option` varchar(100) DEFAULT '' NOT NULL,
                                              PRIMARY KEY (id)
                                            );
                                            ";
            $GLOBALS['TYPO3_DB']->sql_query($sql);
            require_once (GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/application/libraries/SaltCellar.php'));
            $salt = \SaltCellar::getSalt(44, 50);
            $data['join_date'] = date('Y-m-d H:i:s');
            $data['salt'] = $salt;

            require_once (GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/application/libraries/PasswordStorage.php'));
            $data['password'] = \PasswordStorage::create_hash($salt . 'admin');
            $data['username'] = 'admin';
            $GLOBALS['TYPO3_DB']->exec_INSERTquery( 'tx_revslider_domain_model_user', $data);
        }
    }

    /**
     *
     */
    private function instantiateCi()
    {

        ob_start();

        $system_path = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/system');
        $application_folder = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/application');

        chdir(\TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/'));

        if (realpath($system_path) !== FALSE)
            $system_path = realpath($system_path) .DIRECTORY_SEPARATOR;

        $system_path = rtrim($system_path, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR;

        if ( ! is_dir($system_path))
            exit("Your system folder path does not appear to be set correctly. Please open the following file and correct this: ".pathinfo(__FILE__, PATHINFO_BASENAME));

        /** @var Proxy $proxyHelper */
        $proxyHelper = $this->objectManager->get(Proxy::class);
        $proxyHelper->setProxyConstants();

        define('SELF', pathinfo(__FILE__, PATHINFO_BASENAME));
        define('EXT', '.php');
        define('BASEPATH', str_replace("\\", DIRECTORY_SEPARATOR, $system_path));
        define('DOCROOT',  rtrim(\TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('TYPO3_DOCUMENT_ROOT'), DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR);
        define('UPLOADSPATH',  rtrim(\TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName('uploads/tx_revslider/'), DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR);
        define('FCPATH',  rtrim(\TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/'), DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR);
        define('SYSDIR', trim(strrchr(trim(BASEPATH, DIRECTORY_SEPARATOR), DIRECTORY_SEPARATOR), DIRECTORY_SEPARATOR));


        /** @var UriBuilder $backendUriBuilder */
        $backendUriBuilder = $this->objectManager->get(UriBuilder::class);
        define('BACKENDROUTE', $backendUriBuilder->buildUriFromModule('web_RevsliderM1')->getPath());
        define('MODULEQUERY', $backendUriBuilder->buildUriFromModule('web_RevsliderM1')->getQuery());
        define('BASEURI', rtrim(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'), DIRECTORY_SEPARATOR));
        define('CONFIG_BASEURI', rtrim(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'), DIRECTORY_SEPARATOR) . '/typo3conf/ext/revslider/vendor/revslider/');


        if (is_dir($application_folder))
        {
            define('APPPATH', $application_folder.'/');
        }
        else
        {
            if ( ! is_dir(BASEPATH.$application_folder.'/'))
                exit("Your application folder path does not appear to be set correctly. Please open the following file and correct this: ".SELF);

            define('APPPATH', BASEPATH.$application_folder.'/');
        }

        if (file_exists(APPPATH.'config/dev.php'))
            include APPPATH.'config/dev.php';

        if ( ! file_exists(APPPATH.'config/config.php') || ! file_exists(APPPATH.'config/database.php'))
        {
            define('ENVIRONMENT', 'install');
        }
        else
        {
            define('ENVIRONMENT', defined('RS_CI_DEV_MODE') && RS_CI_DEV_MODE ? 'development' : 'production');
        }

        define('RS_DEMO', false);

        if (defined('ENVIRONMENT'))
        {
            switch (ENVIRONMENT)
            {
                case 'development':
                case 'install':
                    error_reporting(E_ALL);
                    break;

                case 'testing':
                case 'production':
                    error_reporting(E_WARNING);
                    break;

                default:
                    exit('The application environment is not set correctly.');
            }
        }
        umask(0);

        require_once BASEPATH.'core/CodeIgniter.php';

         $out = ob_get_contents();
        ob_end_clean();

        $out = str_replace("<head>", '<head><style>html, body {overflow: auto;height: 100%;}</style>', $out);
        echo $out;
    }

}
