<?php
namespace MRU\Revslider\Controller;

use MRU\Revslider\Domain\Helper\Proxy;
use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use TYPO3\CMS\Extensionmanager\Utility\InstallUtility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Manfred Rutschmann <manfred@rutschmann.biz>, Rutschmann.BIZ
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
class InstallController extends ActionController
{

    const APP_VERSION = 'NA';
    const SLIDER_REVISION = '5.3.1.6';
    const SLIDER_PRODUCT = 'revslider_typo3';
    const ACTIVATE_URL = 'http://updates.themepunch.tools/activate.php';
    const CHECK_URL = 'https://updates.themepunch.tools/revslider-typo3-addon/check-license.php';
    CONST REMOTE_URL_INFO	= 'http://updates.themepunch.tools/revslider-typo3-addon/revslider-standalone.php';


    /**
     * @var InstallUtility
     */
    private $extensionInstaller;



    public function __construct(InstallUtility $extensionInstaller = null)
    {
        parent::__construct();
        /** @var ObjectManager $objectManager */
        $objectManager = GeneralUtility::makeInstance(ObjectManager::class);
        /** @var UriBuilder $backendUriBuilder */
        $backendUriBuilder = $objectManager->get(UriBuilder::class);
        define('BACKENDROUTE', $backendUriBuilder->buildUriFromModule('web_RevsliderM1')->getPath());
        define('MODULEQUERY', $backendUriBuilder->buildUriFromModule('web_RevsliderM1')->getQuery());
        define('BASEURI', rtrim(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'), DIRECTORY_SEPARATOR));
        define('CONFIG_BASEURI', rtrim(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'), DIRECTORY_SEPARATOR) . '/typo3conf/ext/revslider/vendor/revslider/');
        define('FCPATH',  rtrim(GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/'), DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR);
        $this->extensionInstaller = $extensionInstaller ?: GeneralUtility::makeInstance(ObjectManager::class)->get(InstallUtility::class);
    }

    /**
     * @param array $response
     */
    public function indexAction($response = null)
    {
        $this->view
            ->assign('license', $response['license'])
            ->assign('response', $response)
            ;
    }

    /**
     * @param array $license
     */
    public function installRevSliderSuiteAction($license)
    {
        $result = $this->checkLicenseKey($license['key']);
        if($result['body'] == 'valid')
        {

            $request = $this->post(self::REMOTE_URL_INFO, array(
                'user-agent' => 'StandAlone/'.self::APP_VERSION.'; '.$this->getUrl(),
                'body' => array(
                    'code' => urlencode($license['key']),
                    'version' => urlencode(self::SLIDER_REVISION),
                )
            ));

            if(array_key_exists('body', $request) && strlen($request['body']) > 0 && $request['body'] !== 'invalid'){
                $data = unserialize($request['body']);

                if($data instanceof \stdClass){

                    $request = $this->post($data->full->download_link, array(
                        'user-agent' => 'StandAlone/'.self::APP_VERSION.'; '.$this->getUrl(),
                        'timeout' => 45,
                        'method' => 'GET'
                    ));

                    if($response = $request['body']) {
                        if ($response !== 'invalid') {

                            $path = GeneralUtility::getFileAbsFileName('uploads/tx_revslider/media/') . 'update' . DIRECTORY_SEPARATOR;
                            $file = $path . $data->full->file_name;
                            GeneralUtility::mkdir_deep($path);
                            GeneralUtility::mkdir(GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/'));

                            $ret = @file_put_contents( $path . $data->full->file_name, $response );
                            if($ret !== false) {



                                if( class_exists('ZipArchive') ){
                                    $zip = new \ZipArchive();
                                    $zipResult = $zip->open($file, \ZipArchive::CREATE);
                                    if ($zipResult === true) {
                                        for($i = 0; $i < $zip->numFiles; $i++) {
                                            $fileName = $zip->getNameIndex($i);
                                            $fileInfo = pathinfo($fileName);
                                            if (strpos($fileName, '_') !== 0 && strpos($fileName, '.') !== 0 && strpos($fileInfo['basename'], '_') !== 0 && strpos($fileInfo['basename'], '.') !== 0) {

                                                if ($fileInfo['dirname'] !== '.' && ! file_exists($path.$fileInfo['dirname'])) {
                                                    $parts = explode('/', $fileInfo['dirname']);
                                                    $dirPath = $path;
                                                    foreach ($parts as $part) {
                                                        $dirPath .= $part . DIRECTORY_SEPARATOR;
                                                        GeneralUtility::mkdir_deep($dirPath);
                                                    }
                                                }
                                                if (substr($fileName, -1) !== '/' && substr($fileName, -1) !== '\\') {
                                                    $targetFile = $path.str_replace('//', DIRECTORY_SEPARATOR, $fileName);
                                                    file_put_contents($targetFile, $zip->getFromName($fileName));
                                                    chmod($targetFile, 0644);
                                                }
                                            }
                                        }
                                        $zip->close();
                                        unlink($file);
                                        $file_info = pathinfo($data->full->file_name);
                                        $update_path = basename($data->full->file_name,'.'.$file_info['extension']);
                                        $this->recurse_move($path . $update_path, FCPATH);
                                        rmdir($path);
                                        $this->callInstaller('emitAfterExtensionInstallSignal', ['tx_revslider']);
                                        $this->redirect('index', 'Backend');
                                    } else {
                                        $this->addFlashMessage('Can not unzip file. Something is bad with the retrieved installer package', 'Error', AbstractMessage::ERROR);
                                    }
                                } else {
                                    $this->addFlashMessage('On your system is zip not installed. Please install the php zip module which is recommend for TYPO3.', 'Error', AbstractMessage::ERROR);
                                }
                            } else {
                                $this->addFlashMessage('The downloaded content was nat valid.', 'Error', AbstractMessage::ERROR);
                            }
                        } else {
                            $this->addFlashMessage('Something goes wrong with the download information. Please try again later.', 'Error', AbstractMessage::ERROR);
                        }
                        $this->addFlashMessage('Something goes wrong with the download information. Please try again later.', 'Error', AbstractMessage::ERROR);
                    }
                } else {
                    $this->addFlashMessage('Something goes wrong with the download information. Please try again later.', 'Error', AbstractMessage::ERROR);
                }
            } else {
                $this->addFlashMessage('We have some problems at the update server. Please try again in a few moments', 'Error', AbstractMessage::ERROR);
                $this->forward('index', NULL, NULL, array('response' => $this->checkLicenseKey($license['key'])));
            }
        } else
        {
            $this->forward('index', NULL, NULL, array('response' => $this->checkLicenseKey($license['key'])));
        }
        $this->forward('index', NULL, NULL, array('response' => $this->checkLicenseKey($license['key'])));
    }

    /**
     * @param $src
     * @param $dst
     */
    private function recurse_move($src, $dst) {
		$src = rtrim($src,'/\\');
		$dst = rtrim($dst,'/\\');
		$dir = opendir($src);
        GeneralUtility::mkdir($dst);
		while(false !== ( $file = readdir($dir)) ) {
			if (( $file != '.' ) && ( $file != '..' )) {
				if ( is_dir($src . '/' . $file) )
				{
					$this->recurse_move($src . '/' . $file,$dst . '/' . $file);
				}
                else
                {
                    rename($src . '/' . $file,$dst . '/' . $file);
                }
            }
        }
        closedir($dir);
        rmdir($src);
    }



    /**
     * @param array $license
     */
    public function checkLicenseKeyAction($license)
    {


        $this->forward('index', NULL, NULL, array('response' => $this->checkLicenseKey($license['key'])));
    }

    /**
     * @param string $key
     * @return array
     */
    private function checkLicenseKey($key)
    {
        $response = $this->post(self::CHECK_URL, array(
            'user-agent' => 'StandAlone/'.self::APP_VERSION.'; '.$this->getUrl(),
            'body' => array(
                'code' => urlencode($key),
                'version' => urlencode(self::SLIDER_REVISION),
                'product' => urlencode(self::SLIDER_PRODUCT)
            )
        ));

        if( !array_key_exists('response', $response) || $response['response']['code'] !== 200 ){
            $this->addFlashMessage('There was an unexpected error while connecting to the license server. Pleas try again in a few moment.', 'Error', AbstractMessage::ERROR);
        } else {

        }

        if($response['body'] == 'valid'){
            $this->addFlashMessage('Your license key is valid. You can proceed with the installation', 'OK', AbstractMessage::OK);
        }elseif($response['body'] == 'exist'){
            $this->addFlashMessage('Your license key is valid, but it is active in a other installation. Please detach your license code in the other installation or purchase a new one.', 'Warning', AbstractMessage::WARNING);
        } else {
            $this->addFlashMessage('Your license key is invalid. Check the key again and start over.', 'Warning', AbstractMessage::WARNING);
        }
        $response['license']['key'] = $key;

        return $response;

    }

    /**
     *
     */
    private function getUrl()
    {
        return BASEURI . BACKENDROUTE . '?' . MODULEQUERY;
    }

    /**
     * @param string $url
     * @param array $args
     * @return array
     */
    private function post($url, $args)
    {
        if ( ! function_exists('curl_version')) {
            return  array(
                'response'	=> array('code' => 0, 'message' => 'cURL library not installed.'),
                'body'		=> false
            );
        }

        $defaults = array(
            'user-agent' => false,
            'headers' => array(),
            'cookies' => array(),
            'httpversion' => CURL_HTTP_VERSION_NONE,
            'timeout' => 30,
            'method' => 'POST',
            'body' => array()
        );
        $args = array_merge($defaults, $args);

        $ch = curl_init();


        /** @var Proxy $proxyHelper */
        $proxyHelper = $this->objectManager->get(Proxy::class);
        $proxyHelper->setProxyConstants();


        if(defined('PROXY_HOST'))
            curl_setopt($ch, CURLOPT_PROXY, PROXY_HOST);

        if(defined('PROXY_PORT'))
            curl_setopt($ch, CURLOPT_PROXYPORT, PROXY_PORT);

        if(defined('PROXY_USER'))
            curl_setopt($ch, CURLOPT_PROXYUSERNAME, PROXY_USER);

        if(defined('PROXY_PASS'))
            curl_setopt($ch, CURLOPT_PROXYPASSWORD, PROXY_PASS);

        $headers = array();
        foreach ($args['headers'] as $key => $value) {
            $headers[] = "$key: $value";
        }
        curl_setopt($ch, CURLOPT_URL, $url);
        if ($args['method'] == 'POST' && $args['body'] && is_array($args['body'])) {
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($args['body'], '', '&'));
        }
        if ($args['user-agent']) {
            curl_setopt($ch, CURLOPT_USERAGENT, $args['user-agent']);
        }
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        if ( !empty($headers)) curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_COOKIE,  implode('; ', $args['cookies']));
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, $args['timeout']);
        curl_setopt($ch, CURLOPT_HTTP_VERSION, $args['httpversion'] == '1.0' ? CURL_HTTP_VERSION_1_0 : ($args['httpversion'] == '1.1' ? CURL_HTTP_VERSION_1_1 : CURL_HTTP_VERSION_NONE));
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_ENCODING, '');

        $output = curl_exec($ch);
        $code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error = curl_errno($ch);
        $message = curl_error($ch);
        curl_close ($ch);
        $result = array(
            'response'	=> array('code' => $code, 'message' => $message, 'error' => $error),
            'body'		=> $output
        );
        return $result;
    }

    /**
     * @param string $method
     * @param array $arguments
     */
    private function callInstaller($method, array $arguments)
    {
        $installer = $this->extensionInstaller;
        call_user_func(
            \Closure::bind(function () use ($installer, $method, $arguments) {
                return call_user_func_array([$installer, $method], $arguments);
            }, null, InstallUtility::class));
    }
}
