<?php
namespace MRU\Revslider\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Manfred Rutschmann <manfred@rutschmann.biz>, Rutschmann.BIZ
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
class SliderController extends ActionController
{


    /**
     * action show
     *
     * @return string
     */
    public function showAction()
    {
        $sliders = explode(',', $this->settings['slider']);
        $content = '';
        if(count($sliders) > 0)
        {
            foreach ($sliders as $slider)
            {
                $content.= $this->getEmbeddedSliderSource($slider);
            }
        }
        $content = $this
            ->filterDoubleJqueryErrorFunctions($content)
            ->moveCSSToPageHeader($content)
            ->moveJSToPageFooter($content)
            ->moveInlinJavaScriptToPageFooter($content)
            ->sanitize_output($content);
        return $content;
    }

    /**
     * @param string $content
     * @return $this
     */
    protected function moveInlinJavaScriptToPageFooter(&$content)
    {

        $jsToAdd = '';
        preg_match_all("/<script((?:(?!src=).)*?)>(.*?)<\\/script>/smix", $content, $dataJs);
        if(count($dataJs[0]) > 0){
            foreach ($dataJs[0] as $jsInline) {
                $jsToAdd.= $jsInline;
                $content = str_replace($jsInline, '', $content);
            }
        }
        $GLOBALS['TSFE']->additionalFooterData[md5($jsToAdd)] = $this->sanitize_output($this->sanitize_JSoutput($jsToAdd));
        return $this;
    }

    /**
     * @param string $content
     * @return $this
     */
    protected function moveJSToPageFooter(&$content)
    {
        preg_match_all("/<script.+?src=[\"']?[^'\"]+['\"]?[^>+]*><\\/script>/", $content, $dataJs);
        if(count($dataJs[0]) > 0){
            $i = 0;
            foreach ($dataJs[0] as $jsFile) {
                $GLOBALS['TSFE']->additionalFooterData[md5($jsFile)] = $jsFile;
                $content = str_replace($jsFile, '', $content);
            }
        }

        return $this;
    }

    /**
     * @param string $content
     * @return $this
     */
    protected function moveCSSToPageHeader(&$content)
    {
        preg_match_all("/<link.+?href=[\"']?[^'\"]+['\"]?[^>+]*>/", $content, $dataCss);
        if(count($dataCss[0]) > 0){
            foreach ($dataCss[0] as $cssFile) {
                $this->response->addAdditionalHeaderData($cssFile);
                $content = str_replace($cssFile, '', $content);
            }
        }
        return $this;
    }

    /**
     * @return string
     */
    protected function getEmbeddedSliderSource($slider)
    {
        $slider = str_replace('tx_revslider_domain_model_sliders_', '', $slider);
        define('CONFIG_BASEURI', rtrim(GeneralUtility::getIndpEnv('TYPO3_SITE_URL'), "/") . '/typo3conf/ext/revslider/vendor/revslider/');
        include_once GeneralUtility::getFileAbsFileName('EXT:revslider/vendor/revslider/embed.php');
        ob_start();
        \RevSliderEmbedder::putRevSlider($slider);
        $content = ob_get_contents();
        ob_clean();
        ob_end_clean();
        return $content;
    }

    /**
     * @param string $content
     * @return $this
     */
    protected function sanitize_output($content)
    {
        $search = array(
            '/\>[^\S ]+/s',  // strip whitespaces after tags, except space
            '/[^\S ]+\</s',  // strip whitespaces before tags, except space
            '/(\s)+/s'       // shorten multiple whitespace sequences
        );

        $replace = array(
            '>',
            '<',
            '\\1'
        );
        $content = preg_replace($search, $replace, $content);
        $content = str_replace(PHP_EOL, ' ', $content);

        return $content;
    }

    /**
     * @param string $content
     * @return $this
     */
    protected function sanitize_JSoutput($content)
    {
        $search = array(
            '/\/\*[\s\S]*?\*\/|([^:|\"]|^)\/\/.*$/m',
        );
        $replace = array(
            '',
        );
        $content = preg_replace($search, $replace, $content);
        return $content;
    }

    protected function filterDoubleJqueryErrorFunctions(&$content){
        preg_match_all("/<script type=\"text\\javascript\">[\\s]*function.*?revslider_showDoubleJqueryError.*?script>/ms", $content, $doubleFunctions);
        if(count($doubleFunctions[0]) > 0){
            foreach ($doubleFunctions[0] as $jsFile) {
                $content = str_replace($jsFile, '', $content);
            }
        }
        return $this;
    }

}
